from pymusicdl.modules.common import common
from pymusicdl.modules.spotify_downloader import spotify_downloader
from pymusicdl.modules.ytDownloader import yt_downloader
from pymusicdl.modules.picker import *


import os, time, glob, sys, subprocess, pafy
# import traceback
from pytube import Playlist
from youtube_title_parse import get_artist_title
from rich.console import Console
from rich.columns import Columns
from rich.table import Table
from rich.panel import Panel

try:
    from .common import common
except:
    from common import common

try:
    from .picker import Picker
except:
    from picker import Picker

class yt_downloader():

    def get_playlist_url(self,plLink):
        """
        Returns all the song links of a playlist. Given the playlist URL.
        """

        pl_links = []
        pl = Playlist(plLink)
        for url in pl.video_urls:
            pl_links.append(url)
        return pl_links

    def download_singles(self):
        """
        Downloads songs based on youtube search. Takes a string as an input.
        """

        cm = common()
        try:
            os.chdir("singles")
        except:
            os.mkdir("singles")
            os.chdir("singles")

        Console().print(Columns([Panel("\ntip:\n  [bold white]* give the name of song and the artist for better search results)\n  * you could paste the youtube video url itself if you're looking for a specific song.[/bold white]\n")]))
        s = input("\nEnter the song name: ")
        print(f"\nLoading search results for {s}...\n")
        s = s.replace(" ","+")

        # Get top 7 video URLs
        video_url = cm.get_url(s)
        j=1
        names = []
        for i in video_url:
            if len(video_url) == 0:
                print("\nThere were no results :(\nmaybe try checking the spelling of the song\n")
                quit()
            try:
                t = pafy.new(i)
                names.append(f"{j} - {t.title}  ({t.duration})")
                j+=1
            except:
                j+=1
                # print(traceback.format_exc())
                # time.sleep(2)
                continue
        
        picker = Picker(names,"Select your choice using arrow keys or press q to quit", indicator=" => ")
        picker.register_custom_handler(ord('q'), lambda picker: exit())
        picker.register_custom_handler(ord('Q'), lambda picker: exit())
        op,c = picker.start()
        
        Console().print(Columns([Panel(f"\nDownload size: [green]{int((pafy.new(video_url[c]).getbestaudio().get_filesize()/1048576)*100)/100} MB[/green]\n")]))
        print()
        
        print("\nWould you like an mp3 or flac conversion?\n")
        Console().rule("[bold]**** Here's a quick comparison on both codec ****[bold]", style="black", align="center")
        print("\n")

        table = Table(show_header=True, header_style="bold cyan")
        table.add_column("Avaliable Codec")
        table.add_column("Bit-rate")
        table.add_column("File Size")
        table.add_column("Remarks")
        table.add_row(
            "mp3", "320kbps (default)","~7.3MB for 3min song","Standard codec with normal experience"
        )
        table.add_row()
        table.add_row(
            "flac", "usually >800kbps (1713kbps while testing, 5x of mp3)","~39MB for 3min song","Takes fair amount of disk space but gives amazing experience"
        )
        Console().print(table)
        Console().rule("\n[bold]Note: this step [red]does not use internet[/red] [bold]\n", style="black", align="center")

        print('\nIf you are confused on what to select, select mp3 (default)')
        z = input("\tEnter\n\t1/flac/f - flac\n\tany key - mp3 : ")

        cm.download_song(video_url[c],'','',z)
        print("\n\n")
        Console().print(Columns([Panel(f"\n    Your song is downloaded in \"[bold cyan]/musicDL downloads/singles[/bold cyan]\" folder on desktop    \n")]))
        print("\n\n")
        time.sleep(3)

        picker = Picker(["Open the song directory","Open the song itself"],"Select your choice using arrow keys or press q to quit", indicator=" => ")
        picker.register_custom_handler(ord('q'), lambda picker: 'qq')
        picker.register_custom_handler(ord('Q'), lambda picker: 'QQ')
        _,op = picker.start()

        if op == 0:
            if sys.platform=='win32' or os.name=='nt':
                os.startfile(".")
            elif sys.platform=='linux' or os.name=='posix':
                subprocess.call(['xdg-open','.'])

        elif op == 1:
            file = pafy.new(video_url[c-1]).title
            a,t = get_artist_title(file)

            if file+".mp3" in os.listdir():

                if sys.platform=='win32' or os.name=='nt':
                    os.startfile(file+".mp3")

                elif sys.platform=='linux' or os.name=='posix':
                    subprocess.call(['xdg-open',file+".mp3"])
                
            elif t+" - "+a+".mp3" in os.listdir():
                if sys.platform=='win32' or os.name=='nt':
                    os.startfile(t+" - "+a+".mp3")

                elif sys.platform=='linux' or os.name=='posix':
                    subprocess.call(['xdg-open',t+" - "+a+".mp3"])
                
            else:
                files = glob.glob("./*")
                song = max(files, key = os.path.getctime)
                if sys.platform=='win32' or os.name=='nt':
                    os.startfile(song)
                elif sys.platform=='linux' or os.name=='posix':
                    subprocess.call(['xdg-open',song])
        else:
            return
        
    
    def download_playlist(self):
        """
        Downloads a playlist of songs given the URL
        """

        cm = common()
        try:
            os.chdir("Playlists")
        except:
            os.mkdir("Playlists")
            os.chdir("Playlists")

        print()
        Console().print(Columns([Panel(f"\n       [bold red]MAKE SURE YOUR PLAYLIST IS PUBLIC[/bold red]\n  [bold red]YOU CAN MAKE IT PRIVATE AFTER DOWNLOADING[/bold red]     \n")]))

        plLink = input("Enter your YouTube playlist URL: ")
        plName = input("Give a name to your playlist: ")

        try:
            os.chdir(plName)
        except:
            os.mkdir(plName)
            os.chdir(plName)

        if "https://www" in plLink:
            plLink = plLink.replace("https://www","https://music")
        
        start_time = time.time()
        try:
            plLinks = self.get_playlist_url(plLink)
        except Exception as e:
            print(f"Something went wrong. Maybe check your URL. Here's the reason from the compiler: {e}")
            print("Exiting the program")
            return
        end_time = time.time()
        print(f"\nTime taken to fetch the URLs from Youtube: %.2f secs"%(end_time-start_time))

        data = 0.0
        print("\nCalculating total download size...\n")
        for i in plLinks:
            try:
                data += pafy.new(i).getbestaudio().get_filesize()
            except:
                # print(traceback.format_exc())
                # time.sleep(2)
                continue
        data = int((data/1048576)*100)/100
        
        Console().print(Columns([Panel(f"\nDownload size: [green]{data} MB[/green]\n")]))
        print()
        
        print("\nWould you like an mp3 or flac conversion?\n")
        Console().rule("[bold]**** Here's a quick comparison on both codec ****[bold]", style="black", align="center")
        print("\n")

        table = Table(show_header=True, header_style="bold cyan")
        table.add_column("Avaliable Codec")
        table.add_column("Bit-rate")
        table.add_column("File Size")
        table.add_column("Remarks")
        table.add_row(
            "mp3", "320kbps (default)","~7.3MB for 3min song","Standard codec with normal experience"
        )
        table.add_row()
        table.add_row(
            "flac", "usually >800kbps (1713kbps while testing, 5x of mp3)","~39MB for 3min song","Takes fair amount of disk space but gives amazing experience"
        )
        Console().print(table)
        Console().rule("\n[bold]Note: this step [red]does not use internet[/red] [bold]\n", style="black", align="center")

        print('\nIf you are confused on what to select, select mp3 (default)')
        z = input("\tEnter\n\t1/flac/f - flac\n\tany key - mp3 : ")

        total_songs = len(plLinks)
        for i in plLinks:
            if sys.platform=='win32' or os.name=='nt':
                os.system("cls")
            elif sys.platform=='linux' or os.name=='posix':
                os.system("clear")
            Console().print("[bold][green]Downloaded songs:[/green][/bold]")
            Console().print(Columns([Panel(''.join(list(''.join(iz + '\n' * (N % 3 == 2) for N, iz in enumerate([ii+" " for ii in user.split()]))))+"\n[b][green]Downloaded[/green][/b]", expand=True) for user in os.listdir()]))
            try:
                cm.download_song(i,"",'',z)
            except Exception:
                # print(traceback.format_exc())
                # time.sleep(2)
                continue
            time.sleep(1)
        downloaded_songs = len(os.listdir())
        if total_songs-downloaded_songs!=0:
            print(f"\n{total_songs-downloaded_songs}/{total_songs} songs were not downloaded due to some error")

        print("\n\n")
        Console().print(Columns([Panel(f"\n     Your playlist is downloaded in \"[bold]/musicDL downloads/Playlists/{plName}[/bold]\" folder on desktop     \n")]))
        print("\n\n")

        op = input("Would you like to open the the playlist? (Y/N) ")
        if op.lower() == "y":
            if sys.platform=='win32' or os.name=='nt':
                os.startfile(".")
            elif sys.platform=='linux' or os.name=='posix':
                subprocess.call(['xdg-open','.'])
        else:
            return


import os, subprocess, sys, logging
import traceback, requests
from rich.console import Console

current_version = ""
latest_version = requests.get(f'https://pypi.org/pypi/pymusicdl/json').json()['info']['version']

try:
    from .modules.ytDownloader import yt_downloader
except:
    from modules.ytDownloader import yt_downloader

try:
    from .modules.spotify_downloader import spotify_downloader 
except:
    from modules.spotify_downloader import spotify_downloader 

try:
    from .modules.picker import Picker
except:
    from modules.picker import Picker

def check_ffmpeg():
    ffmpeg_available = True
    try:
        if sys.platform=='linux' or os.name=='posix':
            subprocess.check_output(['which', 'ffmpeg']) 
            
        if sys.platform=='win32' or os.name=='nt':
            subprocess.check_output(['where', 'ffmpeg'])

    except Exception:
        ffmpeg_available = False
    return ffmpeg_available
    
def create_dir():
    """
    Creates "musicDL downloads" directory on desktop when called. Takes no parameters.
    """
    
    if os.path.isdir(os.path.expanduser("~/Desktop")):
        os.chdir(os.path.expanduser("~/Desktop"))
        
    elif os.path.isdir(os.path.expanduser("~/OneDrive/Desktop")):
        os.chdir(os.path.expanduser("~/OneDrive/Desktop"))
    try:
        os.chdir("musicDL downloads")
    except:
        os.mkdir("musicDL downloads")
        os.chdir("musicDL downloads")

path = os.getcwd()+'/'
def main():
    if sys.platform=='win32' or os.name=='nt':
        os.system("cls")
    elif sys.platform=='linux' or os.name=='posix':
        os.system("clear")
        
    if check_ffmpeg():

        os.chdir(path)

        picker = Picker(["Download a single song","Download a YouTube Playlist","Download from Spotify"],"Select your choice using arrow keys or press q to quit", indicator=" => ")
        picker.register_custom_handler(ord('q'), lambda picker: exit())
        picker.register_custom_handler(ord('Q'), lambda picker: exit())
        _,ch = picker.start()

        Console().rule("\n[bold]Note that you can always quit the program using \"ctrl+c\" shortcut [bold]", style="black", align="center")
        create_dir()
        LOG_FILE = os.path.expanduser("~/Desktop/musicDL downloads/logger.log")
        logging.basicConfig(filename=LOG_FILE, level=logging.ERROR)
        print()
        ch+=1
        
        if ch == 1:
            try:
                yt_downloader().download_singles()
            except Exception as e:
                logging.exception(e)
                print(f"\n\nLooks like something went wrong :(\n{traceback.format_exc()}\n\n")
                print("The error has been logged to logger.log file in 'musicDL downloads' on Desktop for reference")
                print("Take a screenshort and raise an issue in github or send it to the devs\n")
                pass

        elif ch == 2:
            try:
                yt_downloader().download_playlist()
            except Exception as e:
                logging.exception(e)
                print(f"\n\nLooks like something went wrong :(\n{traceback.format_exc()}\n\n")
                print("The error has been logged to logger.log file in 'musicDL downloads' on Desktop for reference")
                print("Take a screenshort and raise an issue in github or send it to the devs\n")
                pass

        elif ch == 3:
            try:
                spotify_downloader().interface()
            except Exception as e:
                logging.exception(e)
                print(f"\n\nLooks like something went wrong :(\n{traceback.format_exc()}\n\n")
                print("The error has been logged to logger.log file in 'musicDL downloads' on Desktop for reference")
                print("Take a screenshort and raise an issue in github or send it to the devs\n")
                pass

        else:
            print("\ninvalid option :(\n")
            main()
        n = input("Do you want to continue? (Y/N): ")
        if n.lower() == 'y':
            main()
        else:
            print("\nSee you later!\n")
    else:
        print("Please install FFMPEG and add it to your environment variable")  
        if sys.platform=='win32' or os.name=='nt':
            print("ffmpeg.exe not found.")

        elif sys.platform=='linux' or os.name=='posix':
            print("Use \n\tsudo apt install ffmpeg\nif you're on debian")


import os, urllib.request, re, subprocess, sys, pafy, time, yt_dlp
from rich.console import Console
from youtube_title_parse import get_artist_title
from urllib.parse import quote

class common():
    
    def __init__(self, spo=False):
        self.spo = spo

    def get_url(self,s):
        """
        Give a video ID as an argument to this function. It returns top n (7 by default) video URLs.
        """

        query = "https://www.youtube.com/results?search_query="+quote(s)
        baseurl = "https://www.youtube.com/watch?v="

        response = urllib.request.urlopen(query)
        html = response.read()
        video_ids = re.findall(r"watch\?v=(\S{11})", html.decode())
        
        urls = []
        if self.spo:
            for i in video_ids:
                try:
                    pafy.new(baseurl+i)
                    urls.append(baseurl+i)
                    return urls
                except:
                    continue
        else:
            j = 0
            for i in video_ids:
                try:
                    pafy.new(baseurl+i)
                    urls.append(baseurl+i)
                    j+=1
                    if j==7:
                        break
                except:
                    continue
            return urls

    def convert(self, old, new, alburl, art, artist, flac=False):
        """ converts any file format to .mp3 or flac with the help of ffmpeg """
        
        if self.spo:
            if not flac:
                subprocess.call(['ffmpeg','-hide_banner','-loglevel', 'quiet','-i',old,'-b:a', '320k','w'+new])
                os.remove(old)
                urllib.request.urlretrieve(alburl, "thumb.png")
                print("Adding metadata")
                subprocess.call(['ffmpeg','-hide_banner','-loglevel', 'quiet','-i','w'+new,'-i','thumb.png','-map','0:0','-map','1:0','-codec','copy','-id3v2_version','3','-metadata:s:v','title="Album cover"','-metadata:s:v','comment="Cover (front)"','-metadata','title='+art,'-metadata','artist='+artist,new])
                os.remove("w"+new)
                os.remove("thumb.png")
            elif flac:
                subprocess.call(['ffmpeg','-hide_banner','-loglevel', 'quiet','-i',old,'-c:a', 'flac','w'+new])
                os.remove(old)
                urllib.request.urlretrieve(alburl, "thumb.png")
                print("Adding metadata")
                subprocess.call(['ffmpeg','-hide_banner','-loglevel','quiet','-i','w'+new, '-i','thumb.png','-sample_fmt', 's32', '-ar', '48000', '-disposition:v', 'attached_pic', '-vsync', '0','-metadata','title='+art,'-metadata','artist='+artist, new])
                os.remove("w"+new)
                os.remove("thumb.png")
            
        else:
            if not flac:
                subprocess.call(["ffmpeg",'-hide_banner','-loglevel','quiet','-i',old,'-b:a', '320k',new])
            elif flac:
                subprocess.call(["ffmpeg",'-hide_banner','-loglevel','quiet','-i',old,'-c:a', 'flac',new])
            os.remove(old)

    # Download the song
    def download_song(self,url, albart, sponame, z): 
        """
        Download the song by passing the video URL as a parameter
        """
        
        try:
            v = pafy.new(url)
        except Exception as e:
            print(f"\nSome error occurred while fetching the details of the song : {e}\n")
            return
        options = {'format': 'bestaudio/best'}
        with yt_dlp.YoutubeDL(options) as ydl:
            info = ydl.extract_info(url, False)
            name = ydl.prepare_filename(info).replace(f" [{info['display_id']}]",'')
            
            check_track_name = ''

            # ---------- GET THE PROCESSED TRACK NAME FOR CHECKING IF IT ALREADY EXISTS IN THE DIRECTORY ----------
            try:

                if self.spo:
                    artist, title = get_artist_title(sponame)   
                else:
                    artist, title = get_artist_title(name)

                for i in os.listdir():
                    if sys.platform=='win32' or os.name=='nt':
                            
                        check_track_name = title+" - "+artist
                        check_track_name = check_track_name.replace("\\","_").replace("/","_").replace(":","_").replace("*","_").replace("?","_").replace("\"","_").replace("<","_").replace(">","_").replace("|","_")
            
                    elif sys.platform=='linux' or os.name=='posix':
                        check_track_name = title+" - "+artist
                        check_track_name = check_track_name.replace("\\"," ").replace("/"," ").replace(":"," ").replace("*"," ").replace("?"," ").replace("\""," ").replace("<"," ").replace(">"," ").replace("|"," ") 
            except TypeError:
                for i in os.listdir():
                    if name in i:
                        check_track_name = name[:len(i) - 1 - i[::-1].index('.')]

            if "_" in check_track_name:
                check_track_name = check_track_name.replace("_"," ")

            # ----------  ----------  ----------  ----------  ----------  ----------  ----------  ----------  ---------- 
            # CHECK IF THE TRACK EXISTS OR NOT

            if any(check_track_name in i for i in os.listdir()):
                exist_track = [string for string in os.listdir() if check_track_name in string][0]
                if ".part" not in exist_track:
                    Console().print(f"\n[bold cyan]{name} track already exists. Skipping the track[/bold cyan]")
                    time.sleep(1)
                    return
                elif ".part" in exist_track:
                    Console().print(f"\n[bold green]{name} track is partly downloaded.[/bold green]")

            # ----------  ----------   END OF CHECKING   ----------  ----------

            Console().print(f"\n[bold green]Downloading {name}[/bold green]")
            ydl.download(url)
            os.rename(ydl.prepare_filename(info), ydl.prepare_filename(info).replace(f" [{info['display_id']}]",''))

        # audio.download()
        
        dirs = os.listdir()

        # ----------  ----------   CONVERT THE DOWNLOADED TRACKS TO MP3 OR FLAC   ----------  ----------
        try:

            if self.spo:
                artist, title = get_artist_title(sponame)   
            else:
                artist, title = get_artist_title(name)

            for i in dirs:
                if sys.platform=='win32' or os.name=='nt':
                    if name.replace("\\","_").replace("/","_").replace(":","_").replace("*","_").replace("?","_").replace("\"","_").replace("<","_").replace(">","_").replace("|","_") in i:                    
                        
                        if z == "1" or z == "flac" or z == "f":
                            track_name = title+" - "+artist+".flac"
                            track_name = track_name.replace("\\","_").replace("/","_").replace(":","_").replace("*","_").replace("?","_").replace("\"","_").replace("<","_").replace(">","_").replace("|","_")
                            print(f"Converting the audio format to flac")
                            self.convert(i, track_name, albart, art=title, artist=artist, flac=True)

                        else:
                            track_name = title+" - "+artist+".mp3"
                            track_name = track_name.replace("\\","_").replace("/","_").replace(":","_").replace("*","_").replace("?","_").replace("\"","_").replace("<","_").replace(">","_").replace("|","_")
                            print(f"Converting the audio format to mp3")
                            self.convert(i, track_name, albart, art=title, artist=artist)

                elif sys.platform=='linux' or os.name=='posix':
                    if name in i:

                        if z == "1" or z == "flac" or z == "f":
                            track_name = title+" - "+artist+".flac"
                            track_name = track_name.replace("\\"," ").replace("/"," ").replace(":"," ").replace("*"," ").replace("?"," ").replace("\""," ").replace("<"," ").replace(">"," ").replace("|"," ")
                            print(f"Converting the audio format to flac")
                            self.convert(i, track_name, albart, art=title, artist=artist, flac=True)
                        
                        else:
                            track_name = title+" - "+artist+".mp3"
                            track_name = track_name.replace("\\"," ").replace("/"," ").replace(":"," ").replace("*"," ").replace("?"," ").replace("\""," ").replace("<"," ").replace(">"," ").replace("|"," ")
                            print(f"Converting the audio format to mp3")
                            self.convert(i, track_name, albart, art=title, artist=artist)

            for i in os.listdir():
                if "_" in i:
                    try:
                        os.rename(i,i.replace("_"," "))
                    except FileExistsError:
                        try:
                            os.rename(i,i.replace("_","  "))
                        except FileExistsError:
                            pass
        except TypeError:
            for i in dirs:
                if name in i:
                    ind1 = len(i) - 1 - i[::-1].index('.')
                    ext = i[ind1:]
                    if ext in i:

                        if z == "1" or z == "flac" or z == "f":
                            track_name = name[:ind1]+".flac"
                            print(f"Converting the audio format to flac")
                            self.convert(i, track_name, albart,art='', artist='', flac=True)

                        else:
                            track_name = name[:ind1]+".mp3"
                            print(f"Converting the audio format to mp3")
                            self.convert(i, track_name, albart,art='', artist='')

            for i in os.listdir():
                if "_" in i:
                    os.rename(i,i.replace("_"," "))


import json, base64, os, requests, time, pafy, sys, subprocess
# import traceback
from rich.console import Console
from rich.columns import Columns
from rich.table import Table
from rich.panel import Panel
try:
    import spotipy 
    from spotipy.oauth2 import SpotifyClientCredentials
except ImportError:
    print("Install spotipy library using 'pip install spotipy'")

try:
    from .common import common
except:
    from common import common

destination = os.getcwd()+'/sec.json'
class spotify_downloader():

    def __init__(self):
        self.ytd =  common(spo=True)
    
    def create_PLdir(self,plName):
        """ creates a playlist directory with the given name """

        try:
            os.chdir("Playlists")
        except:
            os.mkdir("Playlists")
            os.chdir("Playlists")
        try:
            os.chdir(plName)
        except:
            os.mkdir(plName)
            os.chdir(plName)
        
    
    def get_json(self):
        id = '1SFCB1mjcNz3U5X0HZTy4j5kpoMdemKWH'
        URL = "https://docs.google.com/uc?export=download"
        session = requests.Session()
        response = session.get(URL, params = { 'id' : id }, stream = True)
        token = None
        for key, value in response.cookies.items():
            if key.startswith('download_warning'):
                token = value
            else:
                token = None
        if token:
            params = { 'id' : id, 'confirm' : token }
            response = session.get(URL, params = params, stream = True)
        CHUNK_SIZE = 32768
        with open(destination, "wb") as f:
            for chunk in response.iter_content(CHUNK_SIZE):
                if chunk: # filter out keep-alive new chunks
                    f.write(chunk)
    
    def get_credentials(self):
        """ returns a token object after authentication. """

        empty = {}
        with open(destination, 'r') as f:
            data = json.load(f)
        with open(destination,'w') as f:
            json.dump(empty,f)
        for key in data.keys():
            data[key] = base64.b64decode(data[key]).decode('utf8')
        client_id = data['id']
        client_secret = data['secret']
        client_credentials_manager = SpotifyClientCredentials(client_id, client_secret)
        return spotipy.Spotify(client_credentials_manager=client_credentials_manager)

    def get_id(self,url):
        """ returns the playlist/album ID given the playlist URI """

        if "playlist" in url:
            ind = url.index("playlist")
            return url[ind+9:ind+31]
        elif "album" in url:
            ind = url.index("album")
            return url[ind+6:ind+28]

    def get_yt_urls(self,tracks):
        """ returns a list of urls that are fetched successfully and dictionary of urls that were failed to be fetched. """

        urls = []
        for key in tracks.keys():
            try:
                got_url = (key+"+lyrics+"+tracks[key]).replace(' ','+')
                print(f"Fetching youtube URL of {key} - {tracks[key]}")
                urls.append(self.ytd.get_url(got_url)[0])
            except Exception as e:
                print(f"\nCould not fetch the link for \"{key} {tracks[key]}\" because: {e}\nmaybe download it as a single")
        return urls

    def get_playlist_tracks(self, sp, plID):
        """ returns a dictionary of track name as key and artist name as value of a spotify playlist given the spotipy token credentials object and playlist id """

        alburl = []
        together = {}
        tracks = {}
        playlist = {}
        offset = 0
        total = sp.playlist_tracks(plID)['total']
        while total-offset>=0:
            
            playlist = sp.playlist_tracks(plID,offset=offset)
            try:
                for i in range(len(playlist['items'])):
                    if len(playlist['items'][i]['track']['album']['images']) == 0:
                        together[playlist['items'][i]['track']['name']] = [playlist['items'][i]['track']['artists'][0]['name'],'https://virginradio.co.uk/sites/virginradio.co.uk/files/song_cover/20160626/coverart_1.png']
                    else:
                        together[playlist['items'][i]['track']['name']] = [playlist['items'][i]['track']['artists'][0]['name'],playlist['items'][i]['track']['album']['images'][1]['url']]
            except:
                continue

            offset +=100
        for key in together.keys():
            tracks[key] = together[key][0]
            alburl.append(together[key][1])
        return tracks, alburl

    def get_album_tracks(self, sp, alID):
        """ returns a dictionary of track name as key and artist name as value of a spotify album given the spotipy token credentials object and album id """

        album = sp.album_tracks(alID)
        tracks = {}
        alburl = []
        for i in range(len(album['items'])):
            tracks[album['items'][i]['name']] = album['items'][i]['artists'][0]['name']
            alburl.append(sp.track(album['items'][i]['id'])['album']['images'][1]['url'])
        return tracks, alburl

    def download_PL(self,plName, urls, alburl, sponame):
        """ downloads all the audio of URLs """

        self.create_PLdir(plName)
        c=1
        data = 0.0
        print("\nCalculating total download size...\n")
        for i in urls:
            try:
                data += pafy.new(i).getbestaudio().get_filesize()
            except:
                # print(traceback.format_exc())
                # time.sleep(2)
                continue
        data = int((data/1048576)*100)/100
        
        Console().print(Columns([Panel(f"\nDownload size: [green]{data} MB[/green]\n")]))
        print()
        
        print("\nWould you like an mp3 or flac conversion?\n")
        Console().rule("[bold]**** Here's a quick comparison on both codec ****[bold]", style="black", align="center")
        print("\n")

        table = Table(show_header=True, header_style="bold cyan")
        table.add_column("Avaliable Codec")
        table.add_column("Bit-rate")
        table.add_column("File Size")
        table.add_column("Remarks")
        table.add_row(
            "mp3", "320kbps (default)","~7.3MB for 3min song","Standard codec with normal experience"
        )
        table.add_row()
        table.add_row(
            "flac", "usually >800kbps (1713kbps while testing, 5x of mp3)","~39MB for 3min song","Takes fair amount of disk space but gives amazing experience"
        )
        Console().print(table)
        Console().rule("\n[bold]Note: this step [red]does not use internet[/red] [bold]\n", style="black", align="center")

        print('\nIf you are confused on what to select, select mp3 (default)')
        z = input("\tEnter\n\tf - flac\n\tany key - mp3 : ")

        for i,j,k in zip(urls, alburl, sponame):
            if os.name == "nt":
                os.system("cls")
            elif os.name == "posix":
                os.system("clear")
            Console().print("[bold][green]Downloaded songs:[/green][/bold]")
            Console().print(Columns([Panel(''.join(list(''.join(iz + '\n' * (N % 3 == 2) for N, iz in enumerate([ii+" " for ii in user.split()]))))+"\n[b][green]Downloaded[/green][/b]", expand=True) for user in os.listdir()]))
            try:
                self.ytd.download_song(i, j, k, z)
            except:
                # print(traceback.format_exc())
                # time.sleep(2)
                continue
            c+=1

    def interface(self):      
        
        self.get_json()
        sponame = []
        print("\n\tConnecting to spotify...\n")
        sp = self.get_credentials()

        plLink = input("\nEnter the Spotify playlist/album URL: ")
        plName = input("\nGive a name to your playlist: ")

        if "playlist" in plLink:    
            plID = self.get_id(plLink)
            tracks,alburls = self.get_playlist_tracks(sp, plID)
        elif "album" in plLink:
            alID = self.get_id(plLink)
            tracks, alburls = self.get_album_tracks(sp,alID)
        else: 
            print("Invalid link, exiting")
            return

        start_time = time.time()
        Console().print(f"\n[bold]Fetching all the relevant URLs[/bold]")
        urls = self.get_yt_urls(tracks)
        end_time = time.time()
        print(f"Time taken to fetch the URLs from Youtube: %.2f secs\n"%(end_time-start_time))

        Console().print("\n[green]Downloading the songs[/green]\n")
        for i in tracks.keys():
            sponame.append(tracks[i]+" - "+i)

        # download the tracks
        self.download_PL(plName, urls, alburls, sponame)

        # check if all the songs are downloaded
        total_songs = len(urls)
        downloaded_songs = len(os.listdir())
        if total_songs-downloaded_songs!=0:
            print(f"\n{total_songs-downloaded_songs}/{total_songs} songs were not downloaded due to some error")
        
        Console().print(Columns([Panel(f"\n     Your playlist is downloaded in \"[bold]/musicDL downloads/Playlists/{plName}[/bold]\" folder on desktop     \n")]))
        op = input("Would you like to open the the playlist? (Y/N) ")
        if op.lower() == "y":
            if sys.platform=='win32' or os.name=='nt':
                os.startfile(".")
            elif sys.platform=='linux' or os.name=='posix':
                subprocess.call(['xdg-open','.'])
        else:
            return


from distutils.core import setup
import setuptools

with open("README.md","r") as f:
    long_description = f.read()

setuptools.setup(
    name = 'pymusicdl',
    packages = setuptools.find_packages(),
    version = '1.1.1',
    license='MIT',
    description = 'Download spotify and youtube playlists without having to enter any API keys!',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author = 'C Vamshi Krishna',
    author_email = 'cvamshik1@gmail.com',
    url = 'https://github.com/insaiyancvk/pymusicdl/',
    keywords = [
    'spotify', 
    'youtube', 
    'music download',
    'music',
    'youtube download',
    'spotify download'
    ],
    include_package_data=True,
    install_requires=[
      'pytube',
      'requests',
      'rich',
      'spotipy',
      'urllib3',
      'youtube-dl',
      'youtube-title-parse',
      'yt-dlp',
      'windows-curses; platform_system=="Windows"'
      ],
    classifiers=[
      'Development Status :: 5 - Production/Stable',
      'Intended Audience :: Developers',
      'Topic :: Software Development :: Build Tools',
      'License :: OSI Approved :: MIT License',
      'Programming Language :: Python :: 3.8'
    ]
)